@@ -31,7 +31,9 @@ CPU_VARIANTS = {
     "imac":             "VexRiscv_IMAC",
     "imac+debug":       "VexRiscv_IMACDebug",
     "full":             "VexRiscv_Full",
+    "full+cfu":         "VexRiscv_FullCfu",
     "full+debug":       "VexRiscv_FullDebug",
+    "full+cfu+debug":   "VexRiscv_FullCfuDebug",
     "linux":            "VexRiscv_Linux",
     "linux+debug":      "VexRiscv_LinuxDebug",
     "linux+no-dsp":     "VexRiscv_LinuxNoDspFmax",
@@ -58,7 +60,9 @@ GCC_FLAGS = {
     "imac":             "-march=rv32imac   -mabi=ilp32",
     "imac+debug":       "-march=rv32imac   -mabi=ilp32",
     "full":             "-march=rv32im     -mabi=ilp32",
+    "full+cfu":         "-march=rv32im     -mabi=ilp32",
     "full+debug":       "-march=rv32im     -mabi=ilp32",
+    "full+cfu+debug":   "-march=rv32im     -mabi=ilp32",
     "linux":            "-march=rv32ima    -mabi=ilp32",
     "linux+debug":      "-march=rv32ima    -mabi=ilp32",
     "linux+no-dsp":     "-march=rv32ima    -mabi=ilp32",
@@ -97,7 +101,7 @@ class VexRiscv(CPU, AutoCSR):
     gcc_triple           = CPU_GCC_TRIPLE_RISCV32
     linker_output_format = "elf32-littleriscv"
     nop                  = "nop"
-    io_regions           = {0x80000000: 0x80000000} # origin, length
+    io_regions           = {0x80000000: 0x80000000} # Origin, Length
 
     # Memory Mapping.
     @property
@@ -131,9 +135,10 @@ class VexRiscv(CPU, AutoCSR):
 
         # # #
 
+        # CPU Instance.
         self.cpu_params = dict(
-            i_clk                    = ClockSignal(),
-            i_reset                  = ResetSignal() | self.reset,
+            i_clk                    = ClockSignal("sys"),
+            i_reset                  = ResetSignal("sys") | self.reset,
 
             i_externalInterruptArray = self.interrupt,
             i_timerInterrupt         = 0,
@@ -164,12 +169,23 @@ class VexRiscv(CPU, AutoCSR):
             i_dBusWishbone_ERR      = dbus.err
         )
 
+        # Add Timer (Optional).
         if with_timer:
             self.add_timer()
 
+        # Add Debug (Optional).
         if "debug" in variant:
             self.add_debug()
 
+    def set_reset_address(self, reset_address):
+        assert not hasattr(self, "reset_address")
+        self.reset_address = reset_address
+        self.cpu_params.update(i_externalResetVector=Signal(32, reset=reset_address))
+
+    def add_timer(self):
+        self.submodules.timer = VexRiscvTimer()
+        self.cpu_params.update(i_timerInterrupt=self.timer.interrupt)
+
     def add_debug(self):
         debug_reset = Signal()
 
@@ -254,14 +270,63 @@ class VexRiscv(CPU, AutoCSR):
             o_debug_resetOut                = self.o_resetOut
         )
 
-    def set_reset_address(self, reset_address):
-        assert not hasattr(self, "reset_address")
-        self.reset_address = reset_address
-        self.cpu_params.update(i_externalResetVector=Signal(32, reset=reset_address))
+    def add_cfu(self, cfu_filename):
+        # Check CFU presence.
+        if not os.path.exists(cfu_filename):
+            raise OSError(f"Unable to find VexRiscv CFU plugin {cfu_filename}.")
 
-    def add_timer(self):
-        self.submodules.timer = VexRiscvTimer()
-        self.cpu_params.update(i_timerInterrupt=self.timer.interrupt)
+        # CFU Layout.
+        cfu_bus_layout = [
+            ("cmd", [
+                ("valid", 1),
+                ("ready", 1),
+                ("payload", [
+                    ("function_id", 10),
+                    ("inputs_0", 32),
+                    ("inputs_1", 32),
+                ]),
+            ]),
+            ("rsp", [
+                ("valid", 1),
+                ("ready", 1),
+                ("payload", [
+                    ("response_ok", 1),
+                    ("outputs_0", 32),
+                ]),
+            ]),
+        ]
+
+        # CFU Bus.
+        self.cfu_bus = cfu_bus = Record(cfu_bus_layout)
+
+        # Add CFU.
+        self.specials += Instance("Cfu",
+            i_cmd_valid                = cfu_bus.cmd.valid,
+            o_cmd_ready                = cfu_bus.cmd.ready,
+            i_cmd_payload_function_id  = cfu_bus.cmd.payload.function_id,
+            i_cmd_payload_inputs_0     = cfu_bus.cmd.payload.inputs_0,
+            i_cmd_payload_inputs_1     = cfu_bus.cmd.payload.inputs_1,
+            o_rsp_valid                = cfu_bus.rsp.valid,
+            i_rsp_ready                = cfu_bus.rsp.ready,
+            o_rsp_payload_response_ok  = cfu_bus.rsp.payload.response_ok,
+            o_rsp_payload_outputs_0    = cfu_bus.rsp.payload.outputs_0,
+            i_clk                      = ClockSignal("sys"),
+            i_reset                    = ResetSignal("sys"),
+        )
+        self.platform.add_source(cfu_filename)
+
+        # Connect CFU to CPU.
+        self.cpu_params.update(
+            o_CfuPlugin_bus_cmd_valid                = cfu_bus.cmd.valid,
+            i_CfuPlugin_bus_cmd_ready                = cfu_bus.cmd.ready,
+            o_CfuPlugin_bus_cmd_payload_function_id  = cfu_bus.cmd.payload.function_id,
+            o_CfuPlugin_bus_cmd_payload_inputs_0     = cfu_bus.cmd.payload.inputs_0,
+            o_CfuPlugin_bus_cmd_payload_inputs_1     = cfu_bus.cmd.payload.inputs_1,
+            i_CfuPlugin_bus_rsp_valid                = cfu_bus.rsp.valid,
+            o_CfuPlugin_bus_rsp_ready                = cfu_bus.rsp.ready,
+            i_CfuPlugin_bus_rsp_payload_response_ok  = cfu_bus.rsp.payload.response_ok,
+            i_CfuPlugin_bus_rsp_payload_outputs_0    = cfu_bus.rsp.payload.outputs_0,
+        )
 
     @staticmethod
     def add_sources(platform, variant="standard"):
@@ -270,9 +335,24 @@ class VexRiscv(CPU, AutoCSR):
         platform.add_source(os.path.join(vdir, cpu_filename))
 
     def add_soc_components(self, soc, soc_region_cls):
+        # Connect Debug interface to SoC.
         if "debug" in self.variant:
-            soc.bus.add_slave("vexriscv_debug", self.debug_bus, region=soc_region_cls(
-                origin=soc.mem_map.get("vexriscv_debug"), size=0x100, cached=False))
+            soc.bus.add_slave("vexriscv_debug", self.debug_bus, region=
+                soc_region_cls(
+                    origin = soc.mem_map.get("vexriscv_debug"),
+                    size   = 0x100,
+                    cached = False
+                )
+            )
+
+        # Pass I/D Caches info to software.
+        base_variant = str(self.variant.split('+')[0])
+        # DCACHE is present on all variants except minimal and lite.
+        if not base_variant in ["minimal", "lite"]:
+            soc.add_config("CPU_HAS_DCACHE")
+        # ICACHE is present on all variants except minimal.
+        if not base_variant in ["minimal"]:
+            soc.add_config("CPU_HAS_ICACHE")
 
     def use_external_variant(self, variant_filename):
         self.external_variant = True
