@@ -123,6 +123,17 @@ video_timings = {
         "v_sync_offset" : 4,
         "v_sync_width"  : 5,
     },
+    "1920x1200@60Hz": {
+        "pix_clk"       : 148.2e6,
+        "h_active"      : 1920,
+        "h_blanking"    : 80,
+        "h_sync_offset" : 8,
+        "h_sync_width"  : 32,
+        "v_active"      : 1200,
+        "v_blanking"    : 35,
+        "v_sync_offset" : 21,
+        "v_sync_width"  : 8,
+    },
 }
 
 # Video Timing Generator ---------------------------------------------------------------------------
@@ -656,36 +667,9 @@ class VideoFrameBuffer(Module, AutoCSR):
 
 class Open(Signal): pass
 
-# VGA (Generic).
+# Generic (Very Generic PHY supporting VGA/DVI and variations).
 
-class VideoVGAPHY(Module):
-    def __init__(self, pads, clock_domain="sys"):
-        self.sink = sink = stream.Endpoint(video_data_layout)
-
-        # # #
-
-        # Always ack Sink, no backpressure.
-        self.comb += sink.ready.eq(1)
-
-        # Drive VGA Clk (Optional).
-        if hasattr(pads, "clk"):
-            self.comb += pads.clk.eq(ClockSignal(clock_domain))
-
-        # Drive VGA Conrols.
-        self.specials += SDROutput(i=~sink.hsync, o=pads.hsync_n, clk=ClockSignal(clock_domain))
-        self.specials += SDROutput(i=~sink.vsync, o=pads.vsync_n, clk=ClockSignal(clock_domain))
-
-        # Drive VGA Datas.
-        cbits  = len(pads.r)
-        cshift = (8 - cbits)
-        for i in range(cbits):
-            self.specials += SDROutput(i=sink.r[cshift + i], o=pads.r[i], clk=ClockSignal(clock_domain))
-            self.specials += SDROutput(i=sink.g[cshift + i], o=pads.g[i], clk=ClockSignal(clock_domain))
-            self.specials += SDROutput(i=sink.b[cshift + i], o=pads.b[i], clk=ClockSignal(clock_domain))
-
-# DVI (Generic).
-
-class VideoDVIPHY(Module):
+class VideoGenericPHY(Module):
     def __init__(self, pads, clock_domain="sys", with_clk_ddr_output=True):
         self.sink = sink = stream.Endpoint(video_data_layout)
 
@@ -694,19 +678,24 @@ class VideoDVIPHY(Module):
         # Always ack Sink, no backpressure.
         self.comb += sink.ready.eq(1)
 
-        # Drive DVI Clk.
-        if with_clk_ddr_output:
-            self.specials += DDROutput(i1=1, i2=0, o=pads.clk, clk=ClockSignal(clock_domain))
-        else:
-            self.comb += pads.clk.eq(ClockSignal(clock_domain))
+        # Drive Clk.
+        if hasattr(pads, "clk"):
+            if with_clk_ddr_output:
+                self.specials += DDROutput(i1=1, i2=0, o=pads.clk, clk=ClockSignal(clock_domain))
+            else:
+                self.comb += pads.clk.eq(ClockSignal(clock_domain))
 
-        # Drive DVI Controls.
+        # Drive Controls.
         if hasattr(pads, "de"):
-            self.specials += SDROutput(i=sink.de,    o=pads.de,    clk=ClockSignal(clock_domain))
-        self.specials += SDROutput(i=sink.hsync, o=pads.hsync, clk=ClockSignal(clock_domain))
-        self.specials += SDROutput(i=sink.vsync, o=pads.vsync, clk=ClockSignal(clock_domain))
+            self.specials += SDROutput(i=sink.de, o=pads.de, clk=ClockSignal(clock_domain))
+        if hasattr(pads, "hsync_n") and hasattr(pads, "vsync_n"):
+            self.specials += SDROutput(i=~sink.hsync, o=pads.hsync_n, clk=ClockSignal(clock_domain))
+            self.specials += SDROutput(i=~sink.vsync, o=pads.vsync_n, clk=ClockSignal(clock_domain))
+        else:
+            self.specials += SDROutput(i=sink.hsync,  o=pads.hsync,   clk=ClockSignal(clock_domain))
+            self.specials += SDROutput(i=sink.vsync,  o=pads.vsync,   clk=ClockSignal(clock_domain))
 
-        # Drive DVI Datas.
+        # Drive Datas.
         cbits  = len(pads.r)
         cshift = (8 - cbits)
         for i in range(cbits):
@@ -714,6 +703,14 @@ class VideoDVIPHY(Module):
             self.specials += SDROutput(i=sink.g[cshift + i], o=pads.g[i], clk=ClockSignal(clock_domain))
             self.specials += SDROutput(i=sink.b[cshift + i], o=pads.b[i], clk=ClockSignal(clock_domain))
 
+# VGA (Generic).
+
+class VideoVGAPHY(VideoGenericPHY): pass
+
+# DVI (Generic).
+
+class VideoDVIPHY(VideoGenericPHY): pass
+
 # HDMI (Generic).
 
 class VideoHDMI10to1Serializer(Module):
@@ -857,6 +854,79 @@ class VideoS7HDMIPHY(Module):
             self.specials += Instance("OBUFDS", i_I=pad_o, o_O=pad_p, o_OB=pad_n)
 
 
+class VideoS7GTPHDMIPHY(Module):
+    def __init__(self, pads, sys_clk_freq, clock_domain="sys", clk_freq=148.5e6, refclk=None):
+        assert sys_clk_freq >= clk_freq
+        self.sink = sink = stream.Endpoint(video_data_layout)
+
+        # # #
+
+        from liteiclink.serdes.gtp_7series import GTPQuadPLL, GTP
+
+        # Always ack Sink, no backpressure.
+        self.comb += sink.ready.eq(1)
+
+        # Clocking + Differential Signaling.
+        pads_clk = Signal()
+        self.specials += DDROutput(i1=1, i2=0, o=pads_clk, clk=ClockSignal(clock_domain))
+        self.specials += Instance("OBUFDS", i_I=pads_clk, o_O=pads.clk_p, o_OB=pads.clk_n)
+
+        # GTP Quad PLL.
+        if refclk is None:
+            # No RefClk provided, use the Video Clk as GTP RefClk.
+            refclk = ClockSignal(clock_domain)
+        elif isinstance(refclk, Record):
+            # Differential RefCLk provided, add an IBUFDS_GTE2.
+            refclk_se = Signal()
+            self.specials += Instance("IBUFDS_GTE2",
+                i_CEB = 0,
+                i_I   = refclk.p,
+                i_IB  = refclk.n,
+                o_O   = refclk_se
+            )
+            refclk = refclk_se
+        self.submodules.pll = pll = GTPQuadPLL(refclk, clk_freq, 1.485e9)
+
+        # Encode/Serialize Datas.
+        for color in ["r", "g", "b"]:
+            # TMDS Encoding.
+            encoder = ClockDomainsRenamer(clock_domain)(TMDSEncoder())
+            self.submodules += encoder
+            self.comb += encoder.d.eq(getattr(sink, color))
+            self.comb += encoder.c.eq(Cat(sink.hsync, sink.vsync) if color == "r" else 0)
+            self.comb += encoder.de.eq(sink.de)
+
+            # 10:20 (SerDes has a minimal 20:1 Serialization ratio).
+            converter = ClockDomainsRenamer(clock_domain)(stream.Converter(10, 20))
+            self.submodules += converter
+            self.comb += converter.sink.valid.eq(1)
+            self.comb += converter.sink.data.eq(encoder.out)
+
+            # Clock Domain Crossing (video_clk --> gtp_tx)
+            cdc = stream.ClockDomainCrossing([("data", 20)], cd_from=clock_domain, cd_to=f"gtp{color}_tx")
+            self.submodules += cdc
+            self.comb += converter.source.connect(cdc.sink)
+            self.comb += cdc.source.ready.eq(1) # No backpressure.
+
+            # 20:1 Serialization + Differential Signaling.
+            c2d  = {"r": 2, "g": 1, "b": 0}
+            class GTPPads:
+                def __init__(self, p, n):
+                    self.p = p
+                    self.n = n
+            tx_pads = GTPPads(p=getattr(pads, f"data{c2d[color]}_p"), n=getattr(pads, f"data{c2d[color]}_n"))
+            # FIXME: Find a way to avoid RX pads.
+            rx_pads = GTPPads(p=getattr(pads, f"rx{c2d[color]}_p"),   n=getattr(pads, f"rx{c2d[color]}_n"))
+            gtp = GTP(pll, tx_pads, rx_pads=rx_pads, sys_clk_freq=sys_clk_freq,
+                tx_polarity      = 1, # FIXME: Specific to Decklink Mini 4K, make it configurable.
+                tx_buffer_enable = True,
+                rx_buffer_enable = True,
+                clock_aligner    = False
+            )
+            setattr(self.submodules, f"gtp{color}", gtp)
+            self.comb += gtp.tx_produce_pattern.eq(1)
+            self.comb += gtp.tx_pattern.eq(cdc.source.data)
+
 # HDMI (Lattice ECP5).
 
 class VideoECP5HDMIPHY(Module):
