@@ -26,7 +26,6 @@
 #include "jsmn.h"
 
 #include <progress.h>
-#include <spiflash.h>
 
 #include <libliteeth/udp.h>
 #include <libliteeth/tftp.h>
@@ -49,11 +48,13 @@
 
 extern void boot_helper(unsigned long r1, unsigned long r2, unsigned long r3, unsigned long addr);
 
-static void __attribute__((noreturn)) boot(unsigned long r1, unsigned long r2, unsigned long r3, unsigned long addr)
+void __attribute__((noreturn)) boot(unsigned long r1, unsigned long r2, unsigned long r3, unsigned long addr)
 {
-	printf("Executing booted program at 0x%08x\n\n", addr);
+	printf("Executing booted program at 0x%08lx\n\n", addr);
 	printf("--============= \e[1mLiftoff!\e[0m ===============--\n");
+#ifdef CSR_UART_BASE
 	uart_sync();
+#endif
 #ifdef CONFIG_CPU_HAS_INTERRUPT
 	irq_setmask(0);
 	irq_setie(0);
@@ -100,6 +101,8 @@ void romboot(void)
 /* Serial Boot                                                           */
 /*-----------------------------------------------------------------------*/
 
+#ifdef CSR_UART_BASE
+
 static int check_ack(void)
 {
 	int recognized;
@@ -107,7 +110,11 @@ static int check_ack(void)
 
 	timer0_en_write(0);
 	timer0_reload_write(0);
+#ifndef CONFIG_DISABLE_DELAYS
 	timer0_load_write(CONFIG_CLOCK_FREQUENCY/4);
+#else
+	timer0_load_write(0);
+#endif
 	timer0_en_write(1);
 	timer0_update_value_write(1);
 	recognized = 0;
@@ -211,7 +218,7 @@ int serialboot(void)
 				char *writepointer;
 
 				failed = 0;
-				writepointer = (char *) get_uint32(&frame.payload[0]);
+				writepointer = (char *)(uintptr_t) get_uint32(&frame.payload[0]);
 				for(i=4;i<frame.payload_length;i++)
 					*(writepointer++) = frame.payload[i];
 				if (frame.cmd == SFL_CMD_LOAD)
@@ -227,31 +234,6 @@ int serialboot(void)
 				boot(0, 0, 0, addr);
 				break;
 			}
-			case SFL_CMD_FLASH: {
-#if (defined CSR_SPIFLASH_BASE && defined SPIFLASH_PAGE_SIZE)
-				uint32_t addr;
-
-				failed = 0;
-				addr = get_uint32(&frame.payload[0]);
-
-				for (i = 4; i < frame.payload_length; i++) {
-					/* Erase page at sector boundaries before writing */
-					if ((addr & (SPIFLASH_SECTOR_SIZE - 1)) == 0) {
-						erase_flash_sector(addr);
-					}
-					write_to_flash(addr, &frame.payload[i], 1);
-					addr++;
-				}
-				uart_write(SFL_ACK_SUCCESS);
-#endif
-				break;
-			}
-			case SFL_CMD_REBOOT:
-#ifdef CSR_CTRL_RESET_ADDR
-				uart_write(SFL_ACK_SUCCESS);
-				ctrl_reset_write(1);
-#endif
-				break;
 			default:
 				failed++;
 				if(failed == MAX_FAILED) {
@@ -265,37 +247,37 @@ int serialboot(void)
 	return 1;
 }
 
+#endif
+
 /*-----------------------------------------------------------------------*/
 /* Ethernet Boot                                                         */
 /*-----------------------------------------------------------------------*/
 
 #ifdef CSR_ETHMAC_BASE
 
-#ifndef LOCALIP1
-#define LOCALIP1 192
-#define LOCALIP2 168
-#define LOCALIP3 1
-#define LOCALIP4 50
-#endif
-
-#ifndef REMOTEIP1
-#define REMOTEIP1 192
-#define REMOTEIP2 168
-#define REMOTEIP3 1
-#define REMOTEIP4 100
-#endif
-
 #ifndef TFTP_SERVER_PORT
 #define TFTP_SERVER_PORT 69
 #endif
 
-static const unsigned char macadr[6] = {0x10, 0xe2, 0xd5, 0x00, 0x00, 0x00};
+static unsigned char macadr[6] = {0x10, 0xe2, 0xd5, 0x00, 0x00, 0x00};
+
+#ifdef LOCALIP1
+static unsigned int local_ip[4] = {LOCALIP1, LOCALIP2, LOCALIP3, LOCALIP4};
+#else
+static unsigned int local_ip[4] = {192, 168, 1, 50};
+#endif
+
+#ifdef REMOTEIP1
+static unsigned int remote_ip[4] = {REMOTEIP1, REMOTEIP2, REMOTEIP3, REMOTEIP4};
+#else
+static unsigned int remote_ip[4] = {192, 168, 1, 100};
+#endif
 
 static int copy_file_from_tftp_to_ram(unsigned int ip, unsigned short server_port,
 const char *filename, char *buffer)
 {
 	int size;
-	printf("Copying %s to 0x%08x... ", filename, buffer);
+	printf("Copying %s to %p... ", filename, buffer);
 	size = tftp_get(ip, server_port, filename, buffer);
 	if(size > 0)
 		printf("(%d bytes)", size);
@@ -303,6 +285,123 @@ const char *filename, char *buffer)
 	return size;
 }
 
+#ifdef ETH_DYNAMIC_IP
+
+static uint8_t parse_ip(const char * ip_address, unsigned int * ip_to_change)
+{
+	uint8_t n = 0;
+	uint8_t k = 0;
+	uint8_t i;
+	uint8_t size = strlen(ip_address);
+	unsigned int ip_to_set[4];
+	char buf[3];
+
+	if (size < 7 || size > 15) {
+		printf("Error: Invalid IP address length.");
+		return -1;
+	}
+
+	/* Extract numbers from input, check for potential errors */
+	for (i = 0; i < size; i++) {
+		if ((ip_address[i] == '.' && k != 0) || (ip_address[i] == '\n' && i == size - 1)) {
+			ip_to_set[n] = atoi(buf);
+			n++;
+			k = 0;
+			memset(buf, '\0', sizeof(buf));
+		} else if (ip_address[i] >= '0' && ip_address[i] <= '9' && k < 3) {
+			buf[k] = ip_address[i];
+			k++;
+		} else {
+			printf("Error: Invalid IP address format. Correct format is \"X.X.X.X\".");
+			return -1;
+		}
+	}
+	ip_to_set[n] = atoi(buf);
+
+	/* Check if a correct number of numbers was extracted from the input*/
+	if (n != 3) {
+		printf("Error: Invalid IP address format. Correct format is \"X.X.X.X\".");
+		return -1;
+	}
+
+	/* Set the extracted IP address as local or remote ip */
+	for (i = 0; i <= n; i++) {
+		ip_to_change[i] = ip_to_set[i];
+	}
+	return 0;
+}
+
+void set_local_ip(const char * ip_address)
+{
+	if (parse_ip(ip_address, local_ip) == 0) {
+		udp_set_ip(IPTOINT(local_ip[0], local_ip[1], local_ip[2], local_ip[3]));
+		printf("Local IP: %d.%d.%d.%d", local_ip[0], local_ip[1], local_ip[2], local_ip[3]);
+	}
+}
+
+void set_remote_ip(const char * ip_address)
+{
+	if (parse_ip(ip_address, remote_ip) == 0) {
+		printf("Remote IP: %d.%d.%d.%d", remote_ip[0], remote_ip[1], remote_ip[2], remote_ip[3]);
+	}
+}
+
+static uint8_t parse_mac_addr(const char * mac_address)
+{
+	uint8_t n = 0;
+	uint8_t k = 0;
+	uint8_t i;
+	uint8_t size = strlen(mac_address);
+	unsigned int mac_to_set[6];
+	char buf[2];
+
+	if (size != 17) {
+		printf("Error: Invalid MAC address length.");
+		return -1;
+	}
+
+	/* Extract numbers from input, check for potential errors */
+	for (i = 0; i < size; i++) {
+		if ((mac_address[i] == ':' && k != 0) || (mac_address[i] == '\n' && i == size - 1)) {
+			mac_to_set[n] = strtol(buf, NULL, 16);
+			n++;
+			k = 0;
+			memset(buf, '\0', sizeof(buf));
+		} else if (((mac_address[i] >= '0' && mac_address[i] <= '9') ||
+			(mac_address[i] >= 'a' && mac_address[i] <= 'f') ||
+			(mac_address[i] >= 'A' && mac_address[i] <= 'F')) && k < 2) {
+			buf[k] = mac_address[i];
+			k++;
+		} else {
+			printf("Error: Invalid MAC address format. Correct format is \"XX:XX:XX:XX:XX:XX\".");
+			return -1;
+		}
+	}
+	mac_to_set[n] = strtol(buf, NULL, 16);
+
+	/* Check if correct number of numbers was extracted from input */
+	if (n != 5) {
+		printf("Error: Invalid MAC address format. Correct format is \"XX:XX:XX:XX:XX:XX\".");
+		return -1;
+	}
+
+	/* Set the extracted MAC address as macadr */
+	for (i = 0; i <= n; i++) {
+		macadr[i] = mac_to_set[i];
+	}
+	return 0;
+}
+
+void set_mac_addr(const char * mac_address)
+{
+	if (parse_mac_addr(mac_address) == 0) {
+		udp_set_mac(macadr);
+		printf("MAC address : %x:%x:%x:%x:%x:%x", macadr[0], macadr[1], macadr[2], macadr[3], macadr[4], macadr[5]);
+	}
+}
+
+#endif
+
 static void netboot_from_json(const char * filename, unsigned int ip, unsigned short tftp_port)
 {
 	int size;
@@ -388,25 +487,34 @@ static void netboot_from_bin(const char * filename, unsigned int ip, unsigned sh
 	boot(0, 0, 0, MAIN_RAM_BASE);
 }
 
-void netboot(void)
+void netboot(int nb_params, char **params)
 {
 	unsigned int ip;
+	char * filename = NULL;
 
+	if (nb_params > 0 )
+		filename = params[0];
 
 	printf("Booting from network...\n");
-	printf("Local IP : %d.%d.%d.%d\n", LOCALIP1, LOCALIP2, LOCALIP3, LOCALIP4);
-	printf("Remote IP: %d.%d.%d.%d\n", REMOTEIP1, REMOTEIP2, REMOTEIP3, REMOTEIP4);
 
-	ip = IPTOINT(REMOTEIP1, REMOTEIP2, REMOTEIP3, REMOTEIP4);
-	udp_start(macadr, IPTOINT(LOCALIP1, LOCALIP2, LOCALIP3, LOCALIP4));
+	printf("Local IP: %d.%d.%d.%d\n", local_ip[0], local_ip[1], local_ip[2], local_ip[3]);
+	printf("Remote IP: %d.%d.%d.%d\n", remote_ip[0], remote_ip[1], remote_ip[2], remote_ip[3]);
 
-	/* Boot from boot.json */
-	printf("Booting from boot.json...\n");
-	netboot_from_json("boot.json", ip, TFTP_SERVER_PORT);
+	ip = IPTOINT(remote_ip[0], remote_ip[1], remote_ip[2], remote_ip[3]);
+	udp_start(macadr, IPTOINT(local_ip[0], local_ip[1], local_ip[2], local_ip[3]));
 
-	/* Boot from boot.bin */
-	printf("Booting from boot.bin...\n");
-	netboot_from_bin("boot.bin", ip, TFTP_SERVER_PORT);
+	if (filename) {
+		printf("Booting from %s (JSON)...\n", filename);
+		netboot_from_json(filename, ip, TFTP_SERVER_PORT);
+	} else {
+		/* Boot from boot.json */
+		printf("Booting from boot.json...\n");
+		netboot_from_json("boot.json", ip, TFTP_SERVER_PORT);
+
+		/* Boot from boot.bin */
+		printf("Booting from boot.bin...\n");
+		netboot_from_bin("boot.bin", ip, TFTP_SERVER_PORT);
+	}
 
 	/* Boot failed if we are here... */
 	printf("Network boot failed.\n");
@@ -450,7 +558,7 @@ static int copy_image_from_flash_to_ram(unsigned int flash_address, unsigned lon
 
 	length = check_image_in_flash(flash_address);
 	if(length > 0) {
-		printf("Copying 0x%08x to 0x%08x (%d bytes)...\n", flash_address, ram_address, length);
+		printf("Copying 0x%08x to 0x%08lx (%d bytes)...\n", flash_address, ram_address, length);
 		offset = 0;
 		init_progression_bar(length);
 		while (length > 0) {
@@ -522,7 +630,7 @@ static int copy_file_from_sdcard_to_ram(const char * filename, unsigned long ram
 	}
 
 	length = f_size(&file);
-	printf("Copying %s to 0x%08x (%d bytes)...\n", filename, ram_address, length);
+	printf("Copying %s to 0x%08lx (%d bytes)...\n", filename, ram_address, length);
 	init_progression_bar(length);
 	offset = 0;
 	for (;;) {
@@ -697,7 +805,7 @@ static int copy_file_from_sata_to_ram(const char * filename, unsigned long ram_a
 	}
 
 	length = f_size(&file);
-	printf("Copying %s to 0x%08x (%d bytes)...\n", filename, ram_address, length);
+	printf("Copying %s to 0x%08lx (%d bytes)...\n", filename, ram_address, length);
 	init_progression_bar(length);
 	offset = 0;
 	for (;;) {
