@@ -409,3 +409,7 @@ static void sdram_spd_handler(int nb_params, char **params)
 }
 define_command(sdram_spd, sdram_spd_handler, "Read SDRAM SPD EEPROM", LITEDRAM_CMDS);
 #endif
+
+#ifdef SDRAM_DEBUG
+define_command(sdram_debug, sdram_debug, "Run SDRAM debug tests", LITEDRAM_CMDS);
+#endif
