@@ -4,11 +4,13 @@
 #include <stdlib.h>
 
 #include <generated/csr.h>
+#include <generated/soc.h>
 
 #include <libliteeth/mdio.h>
 
 #include "../command.h"
 #include "../helpers.h"
+#include "../boot.h"
 
 /**
  * Command "mdio_write"
@@ -134,3 +136,60 @@ static void mdio_dump_handler(int nb_params, char **params)
 
 define_command(mdio_dump, mdio_dump_handler, "Dump MDIO registers", LITEETH_CMDS);
 #endif
+
+/**
+ * Command "eth_local_ip"
+ *
+ * Set local ip
+ *
+ */
+#ifdef ETH_DYNAMIC_IP
+static void eth_local_ip_handler(int nb_params, char **params)
+{
+    if (nb_params < 1) {
+	printf("eth_local_ip <address>");
+	return;
+    }
+
+    set_local_ip(params[0]);
+}
+define_command(eth_local_ip, eth_local_ip_handler, "Set the local ip address", LITEETH_CMDS);
+#endif
+
+/**
+ * Command "eth_remote_ip"
+ *
+ * Set remote ip.
+ *
+ */
+#ifdef ETH_DYNAMIC_IP
+static void eth_remote_ip_handler(int nb_params, char **params)
+{
+    if (nb_params < 1) {
+	printf("eth_remote_ip <address>");
+	return;
+    }
+
+    set_remote_ip(params[0]);
+}
+define_command(eth_remote_ip, eth_remote_ip_handler, "Set the remote ip address", LITEETH_CMDS);
+#endif
+
+/**
+ * Command "eth_mac_addr"
+ *
+ * Set mac address.
+ *
+ */
+#ifdef ETH_DYNAMIC_IP
+static void eth_mac_addr_handler(int nb_params, char **params)
+{
+    if (nb_params < 1) {
+	printf("eth_mac_addr <address>");
+	return;
+    }
+
+    set_mac_addr(params[0]);
+}
+define_command(eth_mac_addr, eth_mac_addr_handler, "Set the mac address", LITEETH_CMDS);
+#endif
