@@ -12,7 +12,8 @@ from distutils.dir_util import copy_tree
 
 def main():
     parser = argparse.ArgumentParser(description="LiteX Bare Metal Demo App.")
-    parser.add_argument("--build-path", help="Target's build path.", required=True)
+    parser.add_argument("--build-path", help="Target's build path (ex build/board_name).", required=True)
+    parser.add_argument("--with-cxx",   action="store_true", help="Enable CXX support.")
     args = parser.parse_args()
 
     # Create demo directory
@@ -23,7 +24,8 @@ def main():
     copy_tree(src, "demo")
 
     # Compile demo
-    os.system(f"export BUILD_DIR=../{args.build_path} && cd demo && make")
+    build_path = args.build_path if os.path.isabs(args.build_path) else os.path.join("..", args.build_path)
+    os.system(f"export BUILD_DIR={build_path} && {'export WITH_CXX=1 &&' if args.with_cxx else ''} cd demo && make")
 
     # Copy demo.bin
     os.system("cp demo/demo.bin ./")
